'''import all'''
from .qq import qq
from .kuwo import kuwo
from .migu import migu
from .joox import joox
from .xiami import xiami
from .kugou import kugou
from .netease import netease
from .qianqian import qianqian
from .baiduFlac import baiduFlac